@@ -1,37 +1,63 @@
-# tools.py
+# abstract_tool.py
 #
-# Super-class implemented and extended by all tools.
+# Copyright 2018-2020 Romain F. T.
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from gi.repository import Gtk, Gdk
 import cairo
+from gi.repository import Gtk
+from .utilities_overlay import utilities_show_overlay_on_context
 
-from .utilities import utilities_show_overlay_on_context
+class WrongToolIdException(Exception):
+	def __init__(self, expected, actual):
+		# Context: an error message
+		msg = _("Can't start operation: wrong tool id (expected {0}, got {1})")
+		super().__init__(msg.format(expected, actual))
 
-class ToolTemplate():
-	__gtype_name__ = 'ToolTemplate'
+################################################################################
+
+class AbstractAbstractTool():
+	"""Super-class implemented and extended by all tools."""
+
+	__gtype_name__ = 'AbstractAbstractTool'
+	UI_PATH = '/com/github/maoschanz/drawing/tools/ui/'
 
 	def __init__(self, tool_id, label, icon_name, window, **kwargs):
+		self.window = window
+		# The tool's identity
 		self.id = tool_id
-		self.accept_selection = False
-		self.use_color = True
 		self.menu_id = 0
 		self.label = label
 		self.icon_name = icon_name
-		self.tool_width = 10
+		# The options it supports
+		self.accept_selection = False
+		self.use_color = False
+		self.use_operator = False
+		# The tool's state
 		self.cursor_name = 'cell'
-		self.use_size = False
-		self.has_ongoing_operation = False # TODO
-		self.window = window
+		self._ongoing_operation = False
+		# Once everything is set, build the UI
 		self.build_row()
-		self.try_build_panel()
+		self.try_build_pane()
 
 	############################################################################
-	# Actions ##################################################################
+	# Actions for tool's options ###############################################
 
 	def add_tool_action_simple(self, action_name, callback):
 		"""Convenient wrapper method adding a stateless action to the window. It
 		will be named 'action_name' (string) and activating the action will
-		trigger the method 'callback'.""" # XXX shortcut ?
+		trigger the method 'callback'.""" # XXX allow to set shortcuts ?
 		self.window.add_action_simple(action_name, callback, None)
 
 	def add_tool_action_boolean(self, action_name, default):
@@ -47,7 +73,13 @@ class ToolTemplate():
 		self.get_image().set_action_sensitivity(action_name, state)
 
 	def update_actions_state(self):
-		pass
+		self.set_action_sensitivity('main_color', self.use_color)
+		self.set_action_sensitivity('secondary_color', self.use_color)
+		self.set_action_sensitivity('exchange_color', self.use_color)
+		self.set_action_sensitivity('cairo_operator', self.use_operator)
+
+	def get_settings(self):
+		return self.window._settings
 
 	############################################################################
 	# UI #######################################################################
@@ -55,17 +87,17 @@ class ToolTemplate():
 	def show_error(self, error_text):
 		self.window.prompt_message(True, error_text)
 
-	def try_build_panel(self):
+	def try_build_pane(self):
 		pass
 
-	def build_bottom_panel(self):
+	def build_bottom_pane(self):
 		return None
 
 	def add_item_to_menu(self, tools_menu):
 		tools_menu.append(self.label, 'win.active_tool::' + self.id)
 
 	def get_options_model(self):
-		fpath = '/com/github/maoschanz/drawing/tools/ui/tool_' + self.id + '.ui'
+		fpath = self.UI_PATH + 'tool-' + self.id + '.ui'
 		builder = Gtk.Builder.new_from_resource(fpath)
 		return builder.get_object('options-menu')
 
@@ -86,9 +118,7 @@ class ToolTemplate():
 		                                                tooltip_text=self.label)
 		self.row.set_detailed_action_name('win.active_tool::' + self.id)
 		self.label_widget = Gtk.Label(label=self.label)
-		# if self.window.decorations == 'csd-eos': # XXX faisable mais bug laid
-		# 	self.row.set_border_width(6)
-		if self.window._settings.get_boolean('big-icons'):
+		if self.get_settings().get_boolean('big-icons'):
 			size = Gtk.IconSize.LARGE_TOOLBAR
 		else:
 			size = Gtk.IconSize.SMALL_TOOLBAR
@@ -99,9 +129,16 @@ class ToolTemplate():
 		self.row.add(box)
 		self.row.show_all()
 
+	def set_show_label(self, label_visible):
+		self.label_widget.set_visible(label_visible)
+		if label_visible:
+			self.row.get_children()[0].set_halign(Gtk.Align.START)
+		else:
+			self.row.get_children()[0].set_halign(Gtk.Align.CENTER)
+
 	def update_icon_size(self):
 		image = self.row.get_children()[0].get_children()[0]
-		if self.window._settings.get_boolean('big-icons'):
+		if self.get_settings().get_boolean('big-icons'):
 			size = Gtk.IconSize.LARGE_TOOLBAR
 		else:
 			size = Gtk.IconSize.SMALL_TOOLBAR
@@ -120,7 +157,15 @@ class ToolTemplate():
 		pass
 
 	def cancel_ongoing_operation(self):
-		pass
+		self.on_tool_unselected()
+		self.give_back_control(self.accept_selection) # XXX pas sûr
+		self.on_tool_selected()
+		self.restore_pixbuf()
+		self.non_destructive_show_modif()
+		self._ongoing_operation = False
+
+	def has_ongoing_operation(self):
+		return self._ongoing_operation
 
 	def give_back_control(self, preserve_selection):
 		self.restore_pixbuf()
@@ -132,15 +177,24 @@ class ToolTemplate():
 	def do_tool_operation(self, operation):
 		pass
 
+	def start_tool_operation(self, operation):
+		if operation['tool_id'] != self.id:
+			raise WrongToolIdException(operation['tool_id'], self.id)
+		self.restore_pixbuf()
+		self._ongoing_operation = True
+
 	def apply_operation(self, operation):
-		self.get_image().add_pixbuf_to_history()
 		self.simple_apply_operation(operation)
 		self.get_image().update_actions_state()
 		self.get_image().update_history_sensitivity()
 
 	def simple_apply_operation(self, operation):
-		"""Simpler apply_operation, for the "rebuild from history" method."""
-		self.do_tool_operation(operation)
+		"""Simpler apply_operation, for the 'rebuild from history' method."""
+		try:
+			self.do_tool_operation(operation)
+		except Exception as e:
+			self.show_error(str(e))
+		self._ongoing_operation = False
 		self.get_image().add_to_history(operation)
 		self.non_destructive_show_modif()
 
@@ -156,12 +210,6 @@ class ToolTemplate():
 	def get_selection_pixbuf(self):
 		return self.get_selection().get_pixbuf()
 
-	def set_selection_has_been_used(self, state):
-		self.get_selection().has_been_used = state
-
-	def selection_has_been_used(self):
-		return self.get_selection().has_been_used
-
 	############################################################################
 	# Image management #########################################################
 
@@ -171,8 +219,14 @@ class ToolTemplate():
 	def get_surface(self):
 		return self.get_image().get_surface()
 
+	def scale_factor(self):
+		return self.get_image().SCALE_FACTOR
+
+	def get_context(self):
+		return cairo.Context(self.get_surface())
+
 	def get_main_pixbuf(self):
-		return self.get_image().get_main_pixbuf()
+		return self.get_image().main_pixbuf
 
 	def non_destructive_show_modif(self):
 		self.get_image().update()
@@ -198,11 +252,14 @@ class ToolTemplate():
 	def on_draw(self, area, cairo_context):
 		if not self.accept_selection:
 			return
-		# Basic implementation, tools should do it better to fit their needs
-		if self.selection_is_active():
-			self.get_selection().show_selection_on_surface(cairo_context, True)
-			dragged_path = self.get_selection().get_path_with_scroll()
-			utilities_show_overlay_on_context(cairo_context, dragged_path, True)
+		if not self.selection_is_active():
+			return
+		# Basic "wrong" implementation (wtf is "0, 0"), which is never executed
+		# because tools needing to draw an overlay will do it better to fit
+		# their needs.
+		self.get_selection().show_selection_on_surface(cairo_context, True, 0, 0)
+		dragged_path = self.get_selection().get_path_with_scroll(0, 0)
+		utilities_show_overlay_on_context(cairo_context, dragged_path, True)
 
 	############################################################################
 ################################################################################
