@@ -16,145 +16,202 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import cairo
-from gi.repository import Gtk, Gdk, Gio, GdkPixbuf, GLib, Pango
-from .gi_composites import GtkTemplate
+from gi.repository import Gtk, Gdk, Gio, GdkPixbuf, Pango
+from .history_manager import DrHistoryManager
+from .selection_manager import DrSelectionManager
+from .properties import DrPropertiesDialog
 
-from .selection_manager import DrawingSelectionManager
-
-class DrawingMotionBehavior():
+class DrMotionBehavior():
 	HOVER = 0
 	DRAW = 1
 	SLIP = 2
 
+class NoPixbufNoChangeException(Exception):
+	def __init__(self, pb_name):
+		# Context: an error message
+		super().__init__(_("New pixbuf empty, no change applied to %s") % pb_name)
+
 ################################################################################
 
-@GtkTemplate(ui='/com/github/maoschanz/drawing/ui/image.ui')
-class DrawingImage(Gtk.Box):
-	__gtype_name__ = 'DrawingImage'
+@Gtk.Template(resource_path='/com/github/maoschanz/drawing/ui/image.ui')
+class DrImage(Gtk.Box):
+	__gtype_name__ = 'DrImage'
 
-	drawing_area = GtkTemplate.Child()
-	h_scrollbar = GtkTemplate.Child()
-	v_scrollbar = GtkTemplate.Child()
+	_drawing_area = Gtk.Template.Child()
+	_h_scrollbar = Gtk.Template.Child()
+	_v_scrollbar = Gtk.Template.Child()
 
 	CLOSING_PRECISION = 10
+	SCALE_FACTOR = 1.0 # XXX doesn't work well enough to be anything else
 
 	def __init__(self, window, **kwargs):
 		super().__init__(**kwargs)
-		self.init_template()
 		self.window = window
 
 		self.gfile = None
 		self.filename = None
 
-		self.drawing_area.add_events( \
+		self._init_drawing_area()
+
+		self._update_background_color()
+		self.window._settings.connect('changed::ui-background-rgba', \
+		                                          self._update_background_color)
+
+		self._update_zoom_behavior()
+		self.window._settings.connect('changed::ctrl-zoom', \
+		                                             self._update_zoom_behavior)
+
+	def _init_drawing_area(self):
+		self._drawing_area.add_events( \
 			Gdk.EventMask.BUTTON_PRESS_MASK | \
 			Gdk.EventMask.BUTTON_RELEASE_MASK | \
 			Gdk.EventMask.POINTER_MOTION_MASK | \
 			Gdk.EventMask.SMOOTH_SCROLL_MASK | \
 			Gdk.EventMask.ENTER_NOTIFY_MASK | \
 			Gdk.EventMask.LEAVE_NOTIFY_MASK)
-		# Utiliser BUTTON_MOTION_MASK au lieu de POINTER_MOTION_MASK serait
-		# moins complexe mais moins puissant
+		# Using BUTTON_MOTION_MASK instead of POINTER_MOTION_MASK would be less
+		# algorithmically complex but not "powerful" enough.
 
 		# For displaying things on the widget
-		self.drawing_area.connect('draw', self.on_draw)
+		self._drawing_area.connect('draw', self.on_draw)
+
 		# For drawing with tools
-		self.drawing_area.connect('motion-notify-event', self.on_motion_on_area)
-		self.drawing_area.connect('button-press-event', self.on_press_on_area)
-		self.drawing_area.connect('button-release-event', self.on_release_on_area)
+		self._drawing_area.connect('motion-notify-event', self.on_motion_on_area)
+		self._drawing_area.connect('button-press-event', self.on_press_on_area)
+		self._drawing_area.connect('button-release-event', self.on_release_on_area)
+
 		# For scrolling
-		self.drawing_area.connect('scroll-event', self.on_scroll_on_area)
-		self.h_scrollbar.connect('value-changed', self.on_scrollbar_value_change)
-		self.v_scrollbar.connect('value-changed', self.on_scrollbar_value_change)
+		self._drawing_area.connect('scroll-event', self.on_scroll_on_area)
+		self._h_scrollbar.connect('value-changed', self.on_scrollbar_value_change)
+		self._v_scrollbar.connect('value-changed', self.on_scrollbar_value_change)
+
 		# For the cursor
-		self.drawing_area.connect('enter-notify-event', self.on_enter_image)
-		self.drawing_area.connect('leave-notify-event', self.on_leave_image)
+		self._drawing_area.connect('enter-notify-event', self.on_enter_image)
+		self._drawing_area.connect('leave-notify-event', self.on_leave_image)
+
+	def _update_background_color(self, *args):
+		rgba = self.window._settings.get_strv('ui-background-rgba')
+		self._bg_rgba = (float(rgba[0]), float(rgba[1]), \
+		                                         float(rgba[2]), float(rgba[3]))
+		# We remember this data here for performance, since it's used by the
+		# on_draw method which is called a lot.
+
+	def _update_zoom_behavior(self, *args):
+		self._ctrl_to_zoom = self.window._settings.get_boolean('ctrl-zoom')
 
 	############################################################################
 	# Image initialization #####################################################
 
-	def init_image(self):
+	def init_image_common(self):
 		"""Part of the initialization common to both a new blank image and an
 		opened image."""
 		self._is_pressed = False
 
 		# Zoom and scroll initialization
-		self.zoom_level = 1.0
 		self.scroll_x = 0
 		self.scroll_y = 0
-		self.motion_behavior = DrawingMotionBehavior.HOVER
+		self.set_zoom_level(100) # will do `self.zoom_level = 1.0`
+		self.motion_behavior = DrMotionBehavior.HOVER
 		self.press2_x = 0.0
 		self.press2_y = 0.0
 		self.drag_scroll_x = 0.0
 		self.drag_scroll_y = 0.0
 
 		# Selection initialization
-		self.selection = DrawingSelectionManager(self)
-		
+		self.selection = DrSelectionManager(self)
+
 		# History initialization
-		self.undo_history = []
-		self.redo_history = []
-		self._is_saved = True
-		self.window.lookup_action('undo').set_enabled(False)
-		self.window.lookup_action('redo').set_enabled(False)
+		self._history = DrHistoryManager(self)
+		self.set_action_sensitivity('undo', False)
+		self.set_action_sensitivity('redo', False)
 
 	def init_background(self, width, height, background_rgba):
 		r = float(background_rgba[0])
 		g = float(background_rgba[1])
 		b = float(background_rgba[2])
 		a = float(background_rgba[3])
-		self.initial_operation = {
+		op = {
 			'tool_id': None,
 			'pixbuf': None,
-			'red': r,
-			'green': g,
-			'blue': b,
-			'alpha': a,
-			'width': width,
-			'height': height
+			'red': r, 'green': g, 'blue': b, 'alpha': a,
+			'width': width, 'height': height
 		}
-		self.init_image()
+		self.init_image_common()
+		self._history.initial_operation = op
 		self.restore_first_pixbuf()
 
 	def try_load_pixbuf(self, pixbuf):
-		if not pixbuf.get_has_alpha():
-			pixbuf = pixbuf.add_alpha(False, 0, 0, 0)
-		self.initial_operation = {
-			'tool_id': None,
-			'pixbuf': pixbuf,
-			'red': 0.0, 'green': 0.0, 'blue': 0.0, 'alpha': 0.0,
-			'width': pixbuf.get_width(), 'height': pixbuf.get_height()
-		}
-		self.main_pixbuf = pixbuf
-		self.init_image()
+		self.init_image_common()
+		self._load_pixbuf_common(pixbuf)
 		self.restore_first_pixbuf()
 		self.update_title()
 
+	def _new_blank_pixbuf(self, w, h):
+		return GdkPixbuf.Pixbuf.new(GdkPixbuf.Colorspace.RGB, True, 8, w, h)
+
 	def restore_first_pixbuf(self):
-		"""Set the first saved pixbuf as the main_pixbuf. This is used to
-		rebuild the picture from its history."""
-		pixbuf = self.initial_operation['pixbuf']
-		width = self.initial_operation['width']
-		height = self.initial_operation['height']
-		self.temp_pixbuf = GdkPixbuf.Pixbuf.new(GdkPixbuf.Colorspace.RGB, True, 8, 1, 1)
+		"""Set the last saved pixbuf from the history as the main_pixbuf. This
+		is used to rebuild the picture from its history."""
+		last_saved_pixbuf_op = self._history.get_last_saved_state()
+
+		# restore the state found in the history
+		pixbuf = last_saved_pixbuf_op['pixbuf']
+		width = last_saved_pixbuf_op['width']
+		height = last_saved_pixbuf_op['height']
+		self.set_temp_pixbuf(self._new_blank_pixbuf(1, 1))
 		self.selection.init_pixbuf()
 		self.surface = cairo.ImageSurface(cairo.Format.ARGB32, width, height)
 		if pixbuf is None:
-			r = self.initial_operation['red']
-			g = self.initial_operation['green']
-			b = self.initial_operation['blue']
-			a = self.initial_operation['alpha']
-			self.main_pixbuf = GdkPixbuf.Pixbuf.new(GdkPixbuf.Colorspace.RGB, \
-			                                             True, 8, width, height)
+			# no pixbuf in the operation: the restored state is a blank one
+			r = last_saved_pixbuf_op['red']
+			g = last_saved_pixbuf_op['green']
+			b = last_saved_pixbuf_op['blue']
+			a = last_saved_pixbuf_op['alpha']
+			self.set_main_pixbuf(self._new_blank_pixbuf(width, height))
 			cairo_context = cairo.Context(self.surface)
 			cairo_context.set_source_rgba(r, g, b, a)
 			cairo_context.paint()
 			self.update()
 			self.set_surface_as_stable_pixbuf()
 		else:
-			self.main_pixbuf = self.initial_operation['pixbuf'].copy()
+			self.set_main_pixbuf(last_saved_pixbuf_op['pixbuf'].copy())
 			self.use_stable_pixbuf()
 
+	############################################################################
+
+	def _load_pixbuf_common(self, pixbuf):
+		if not pixbuf.get_has_alpha():
+			pixbuf = pixbuf.add_alpha(False, 255, 255, 255)
+		background_rgba = self.window._settings.get_strv('default-rgba')
+		r = float(background_rgba[0])
+		g = float(background_rgba[1])
+		b = float(background_rgba[2])
+		a = float(background_rgba[3])
+		op = {
+			'tool_id': None,
+			'pixbuf': pixbuf,
+			'red': r, 'green': g, 'blue': b, 'alpha': a,
+			'width': pixbuf.get_width(), 'height': pixbuf.get_height()
+		}
+		self._history.initial_operation = op
+		self.set_main_pixbuf(pixbuf)
+
+	def reload_from_disk(self):
+		"""Safely reloads the image from the disk."""
+		if self.gfile is None:
+			# XXX no, the action shouldn't be active in the first place
+			isnt_saved = not self.window.saving_manager.confirm_save_modifs()
+			if isnt_saved or self.get_file_path() is None:
+				self.window.prompt_message(True, \
+				            _("Can't reload a never-saved file from the disk."))
+				return
+		disk_pixbuf = GdkPixbuf.Pixbuf.new_from_file(self.get_file_path())
+		self._load_pixbuf_common(disk_pixbuf)
+		self.window.set_picture_title(self.update_title())
+		self.use_stable_pixbuf()
+		self.update()
+		self.remember_current_state()
+
 	def try_load_file(self, gfile):
 		self.gfile = gfile
 		pixbuf = GdkPixbuf.Pixbuf.new_from_file(self.get_file_path())
@@ -177,7 +234,7 @@ class DrawingImage(Gtk.Box):
 		event_box.connect('button-press-event', self.on_tab_title_clicked)
 		# These widgets are packed in a regular box, which is returned.
 		tab_title = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, expand=True)
-		if self.window.decorations == 'csd-eos':
+		if self.window.deco_layout == 'he':
 			tab_title.pack_start(btn, expand=False, fill=False, padding=0)
 			tab_title.pack_end(event_box, expand=True, fill=True, padding=0)
 		else:
@@ -185,7 +242,7 @@ class DrawingImage(Gtk.Box):
 			tab_title.pack_end(btn, expand=False, fill=False, padding=0)
 		tab_title.show_all()
 		return tab_title
-
+	
 	def on_tab_title_clicked(self, widget, event_button):
 		if event_button.type == Gdk.EventType.BUTTON_PRESS \
 		and event_button.button == Gdk.BUTTON_MIDDLE:
@@ -195,8 +252,8 @@ class DrawingImage(Gtk.Box):
 
 	def update_title(self):
 		main_title = self.get_filename_for_display()
-		if not self._is_saved:
-			main_title = '*' + main_title
+		if not self.is_saved():
+			main_title = "*" + main_title
 		self.set_tab_label(main_title)
 		return main_title
 
@@ -215,6 +272,7 @@ class DrawingImage(Gtk.Box):
 			self.selection.reset(False)
 			self.main_pixbuf = None
 			self.temp_pixbuf = None
+			self._history.empty_history()
 			return True
 		else:
 			return False
@@ -228,56 +286,53 @@ class DrawingImage(Gtk.Box):
 		else:
 			return self.gfile.get_path()
 
-	############################################################################
-	# Wrappers for window methods related to tools #############################
-
-	def active_tool(self):
-		return self.window.active_tool()
-
-	def get_right_rgba(self):
-		return self.window.color_popover_r.color_widget.get_rgba()
-
-	def get_left_rgba(self):
-		return self.window.color_popover_l.color_widget.get_rgba()
+	def show_properties(self):
+		DrPropertiesDialog(self.window, self)
 
 	############################################################################
 	# History management #######################################################
 
-	def try_undo(self, *args):
-		self.active_tool().cancel_ongoing_operation()
-		if len(self.undo_history) != 0:
-			self.redo_history.append(self.undo_history.pop())
-		# TODO implement operation_is_ongoing
-		# if self.window.operation_is_ongoing():
-		# 	self.active_tool().cancel_ongoing_operation()
-		# elif len(self.undo_history) != 0:
-		# 	self.redo_history.append(self.undo_history.pop())
+	def try_undo(self):
+		self._history.try_undo()
 
-	def try_redo(self, *args):
-		self.undo_history.append(self.redo_history.pop())
+	def try_redo(self):
+		self._history.try_redo()
+
+	def is_saved(self):
+		return self._history.get_saved()
+
+	def remember_current_state(self):
+		self._history.add_state(self.main_pixbuf.copy())
 
 	def update_history_sensitivity(self):
-		can_undo = ( len(self.undo_history) != 0 ) or self.window.operation_is_ongoing()
-		self.window.lookup_action('undo').set_enabled(can_undo)
-		self.window.lookup_action('redo').set_enabled(len(self.redo_history) != 0)
+		self.set_action_sensitivity('undo', self._history.can_undo())
+		self.set_action_sensitivity('redo', self._history.can_redo())
+		# self.update_history_actions_labels()
 
-	def add_operation_to_history(self, operation):
-		self._is_saved = False
-		self.undo_history.append(operation)
-		self.update_history_sensitivity()
+	def add_to_history(self, operation):
+		self._history.add_operation(operation)
 
-	def on_tool_finished(self): # XXX malgré son nom, cette méthode est appelée
-		                        # (uniquement) par [Tool].apply_to_pixbuf
-		#self.redo_history = []
-		self.update_history_sensitivity()
-		self.update()
-		self.set_surface_as_stable_pixbuf() # XXX ici ou dans [Tool].apply_operation ?
-		self.update_actions_state()
+	def should_replace(self):
+		if self._history.can_undo():
+			return False
+		return not self._history.has_initial_pixbuf()
+
+	def get_initial_rgba(self):
+		operation = self._history.initial_operation
+		r = operation['red']
+		g = operation['green']
+		b = operation['blue']
+		a = operation['alpha']
+		return Gdk.RGBA(red=r, green=g, blue=b, alpha=a)
+
+	############################################################################
+	# Misc ? ###################################################################
 
 	def set_action_sensitivity(self, action_name, state):
 		self.window.lookup_action(action_name).set_enabled(state)
 
 	def update_actions_state(self):
+		# XXX shouldn't it be done by the selection_manager?
 		state = self.selection.is_active
 		self.set_action_sensitivity('unselect', state)
 		self.set_action_sensitivity('select_all', not state)
@@ -288,58 +343,64 @@ class DrawingImage(Gtk.Box):
 		self.set_action_sensitivity('new_tab_selection', state)
 		self.active_tool().update_actions_state()
 
-	def should_replace(self):
-		if len(self.undo_history) > 0:
-			return False
-		return self.initial_operation['pixbuf'] is None
+	def active_tool(self):
+		return self.window.active_tool()
 
 	############################################################################
 	# Drawing area, main pixbuf, and surface management ########################
 
 	def on_draw(self, area, cairo_context):
-		"""Signal callback. Executed when self.drawing_area is redrawn."""
+		"""Signal callback. Executed when self._drawing_area is redrawn."""
+		# Background color
+		cairo_context.set_source_rgba(*self._bg_rgba)
+		cairo_context.paint()
+
+		# Zoom level
 		cairo_context.scale(self.zoom_level, self.zoom_level)
+
+		# TODO transform tools may appreciate to draw *before* the image
+
+		# Image (with scroll position)
 		cairo_context.set_source_surface(self.get_surface(), \
 		                                 -1 * self.scroll_x, -1 * self.scroll_y)
 		cairo_context.paint()
 
+		# What the tool is painting
 		self.active_tool().on_draw(area, cairo_context)
 
 	def on_press_on_area(self, area, event):
 		"""Signal callback. Executed when a mouse button is pressed on
-		self.drawing_area, if the button is the mouse wheel the colors are
+		self._drawing_area, if the button is the mouse wheel the colors are
 		exchanged, otherwise the signal is transmitted to the selected tool."""
 		if self._is_pressed:
 			return
 		event_x, event_y = self.get_event_coords(event)
 		if event.button == 2:
-			self.motion_behavior = DrawingMotionBehavior.SLIP
+			self.motion_behavior = DrMotionBehavior.SLIP
 			self.press2_x = event_x
 			self.press2_y = event_y
 			self.drag_scroll_x = event_x
 			self.drag_scroll_y = event_y
 			return
-		self.motion_behavior = DrawingMotionBehavior.DRAW
-		self.active_tool().on_press_on_area(event, self.surface, \
-		                            self.window.thickness_spinbtn.get_value(), \
-		          self.get_left_rgba(), self.get_right_rgba(), event_x, event_y)
+		self.motion_behavior = DrMotionBehavior.DRAW
+		self.active_tool().on_press_on_area(event, self.surface, event_x, event_y)
 		self._is_pressed = True
 
 	def on_motion_on_area(self, area, event):
 		"""Signal callback. Executed when the mouse pointer moves upon
-		self.drawing_area, the signal is transmitted to the selected tool.
+		self._drawing_area, the signal is transmitted to the selected tool.
 		If a button (not the mouse wheel) is pressed, the tool's method should
 		have an effect on the image, otherwise it shouldn't change anything
 		except the mouse cursor icon for example."""
 		event_x, event_y = self.get_event_coords(event)
-		if self.motion_behavior == DrawingMotionBehavior.HOVER:
-			# XXX ça apprécierait sans doute d'avoir direct les bonnes coordonnées ?
+		if self.motion_behavior == DrMotionBehavior.HOVER:
+			# TODO ça apprécierait sans doute d'avoir direct les bonnes coordonnées ?
 			self.active_tool().on_unclicked_motion_on_area(event, self.surface)
-		elif self.motion_behavior == DrawingMotionBehavior.DRAW:
+		elif self.motion_behavior == DrMotionBehavior.DRAW:
 			# implicitely impossible if not self._is_pressed
 			self.active_tool().on_motion_on_area(event, self.surface, event_x, event_y)
-			self.update()
-		else: # self.motion_behavior == DrawingMotionBehavior.SLIP:
+			self.update() # <<< comment this for better perfs
+		else: # self.motion_behavior == DrMotionBehavior.SLIP:
 			delta_x = int(self.drag_scroll_x - event_x)
 			delta_y = int(self.drag_scroll_y - event_y)
 			self.add_deltas(delta_x, delta_y, 0.8)
@@ -348,22 +409,26 @@ class DrawingImage(Gtk.Box):
 
 	def on_release_on_area(self, area, event):
 		"""Signal callback. Executed when a mouse button is released on
-		self.drawing_area, if the button is not the signal is transmitted to the
-		selected tool."""
-		if self.motion_behavior == DrawingMotionBehavior.SLIP:
+		self._drawing_area, if the button is not the signal is transmitted to
+		the selected tool."""
+		if self.motion_behavior == DrMotionBehavior.SLIP:
 			if abs(self.press2_x - self.drag_scroll_x) < self.CLOSING_PRECISION \
 			and abs(self.press2_y - self.drag_scroll_y) < self.CLOSING_PRECISION:
-				self.window.exchange_colors()
-			self.motion_behavior = DrawingMotionBehavior.HOVER
+				self.window.options_manager.on_middle_click()
+			self.motion_behavior = DrMotionBehavior.HOVER
 			return
-		self.motion_behavior = DrawingMotionBehavior.HOVER
+		self.motion_behavior = DrMotionBehavior.HOVER
 		event_x, event_y = self.get_event_coords(event)
 		self.active_tool().on_release_on_area(event, self.surface, event_x, event_y)
 		self.window.set_picture_title()
 		self._is_pressed = False
 
 	def update(self):
-		self.drawing_area.queue_draw()
+		# print('image.py: _drawing_area.queue_draw')
+		# TODO immensément utilisée, mais qui ne scale pas : ça gêne clairement
+		# l'utilisation pour les trop grandes images, il faudrait n'update que
+		# la partie qui change, ou au pire que la partie affichée
+		self._drawing_area.queue_draw()
 
 	def get_surface(self):
 		return self.surface
@@ -375,21 +440,21 @@ class DrawingImage(Gtk.Box):
 		self.window.set_cursor(False)
 
 	def post_save(self):
-		self._is_saved = True
 		self.use_stable_pixbuf()
 		self.update()
 
 	def set_surface_as_stable_pixbuf(self):
-		# print('image/379: set_surface_as_stable_pixbuf')
+		# print('image.py: set_surface_as_stable_pixbuf')
 		self.main_pixbuf = Gdk.pixbuf_get_from_surface(self.surface, 0, 0, \
 		                    self.surface.get_width(), self.surface.get_height())
 
 	def use_stable_pixbuf(self):
-		# print('image/384: use_stable_pixbuf')
+		# print('image.py: use_stable_pixbuf')
+		# TODO immensément utilisée, mais qui ne scale pas : ça gêne clairement
+		# l'utilisation pour les trop grandes images, il faudrait n'update que
+		# la partie qui change, ou au pire que la partie affichée
 		self.surface = Gdk.cairo_surface_create_from_pixbuf(self.main_pixbuf, 0, None)
-
-	def get_main_pixbuf(self):
-		return self.main_pixbuf
+		self.surface.set_device_scale(self.SCALE_FACTOR, self.SCALE_FACTOR)
 
 	def get_pixbuf_width(self):
 		return self.main_pixbuf.get_width()
@@ -398,33 +463,63 @@ class DrawingImage(Gtk.Box):
 		return self.main_pixbuf.get_height()
 
 	def set_main_pixbuf(self, new_pixbuf):
+		"""Safely set a pixbuf as the main one (not used everywhere internally
+		in image.py, but it's normal)."""
 		if new_pixbuf is None:
-			return False
+			raise NoPixbufNoChangeException('main_pixbuf')
 		else:
 			self.main_pixbuf = new_pixbuf
-			return True
-
-	def get_widget_width(self):
-		return self.drawing_area.get_allocated_width()
-
-	def get_widget_height(self):
-		return self.drawing_area.get_allocated_height()
 
 	############################################################################
 	# Temporary pixbuf management ##############################################
 
 	def set_temp_pixbuf(self, new_pixbuf):
 		if new_pixbuf is None:
-			# XXX maybe throw something instead
-			print("new_pixbuf is None, no change to temp_pixbuf will be applied")
+			raise NoPixbufNoChangeException('temp_pixbuf')
 		else:
 			self.temp_pixbuf = new_pixbuf
 
 	def reset_temp(self):
-		self.temp_pixbuf = GdkPixbuf.Pixbuf.new(GdkPixbuf.Colorspace.RGB, True, 8, 1, 1)
+		self.set_temp_pixbuf(self._new_blank_pixbuf(1, 1))
 		self.use_stable_pixbuf()
 		self.update()
 
+	############################################################################
+	# Interaction with the minimap #############################################
+
+	def get_widget_width(self):
+		return self._drawing_area.get_allocated_width()
+
+	def get_widget_height(self):
+		return self._drawing_area.get_allocated_height()
+
+	def get_mini_pixbuf(self, preview_size):
+		mpb_width = self.get_pixbuf_width()
+		mpb_height = self.get_pixbuf_height()
+		if mpb_height > mpb_width:
+			mw = preview_size * (mpb_width/mpb_height)
+			mh = preview_size
+		else:
+			mw = preview_size
+			mh = preview_size * (mpb_height/mpb_width)
+		return self.main_pixbuf.scale_simple(mw, mh, GdkPixbuf.InterpType.TILES)
+
+	def get_show_overlay(self):
+		mpb_width = self.get_pixbuf_width()
+		mpb_height = self.get_pixbuf_height()
+		show_x = self.get_widget_width() < mpb_width * self.zoom_level
+		show_y = self.get_widget_height() < mpb_height * self.zoom_level
+		show_overlay = show_x or show_y
+		return show_overlay
+
+	def get_minimap_ratio(self, mini_width):
+		return mini_width/self.get_pixbuf_width()
+
+	def get_visible_size(self):
+		visible_width = int(self.get_widget_width() / self.zoom_level)
+		visible_height = int(self.get_widget_height() / self.zoom_level)
+		return visible_width, visible_height
+
 	############################################################################
 	# Scroll and zoom levels ###################################################
 
@@ -436,21 +531,62 @@ class DrawingImage(Gtk.Box):
 		event_y = self.scroll_y + (event.y / self.zoom_level)
 		return event_x, event_y
 
-	def on_scroll_on_area(self, area, event):
-		self.add_deltas(event.delta_x, event.delta_y, 10)
+	def get_corrected_coords(self, x1, x2, y1, y2, with_selection, with_zoom):
+		"""Do whatever coordinates conversions are needed by tools like `crop`
+		and `scale` to display things (selection pixbuf, mouse cursors on hover,
+		etc.) correctly enough."""
+		w = x2 - x1
+		h = y2 - y1
+		x1 = x1 - self.scroll_x
+		y1 = y1 - self.scroll_y
+		if with_selection:
+			x1 += self.selection.selection_x
+			y1 += self.selection.selection_y
+		x2 = x1 + w
+		y2 = y1 + h
+		if with_zoom:
+			x1 *= self.zoom_level
+			x2 *= self.zoom_level
+			y1 *= self.zoom_level
+			y2 *= self.zoom_level
+		return x1, x2, y1, y2
 
-	def get_dragged_selection_path(self):
-		return self.selection.get_path_with_scroll(self.scroll_x, self.scroll_y)
+	def get_nineths_sizes(self, apply_to_selection, x1, y1):
+		"""Returns the sizes of the 'nineths' of the image used for example by
+		'scale' or 'crop' to decide the cursor they'll show."""
+		height = self.temp_pixbuf.get_height()
+		width = self.temp_pixbuf.get_width()
+		if not apply_to_selection:
+			x1 = 0
+			y1 = 0
+		# width_left, width_right, height_top, height_bottom
+		wl, wr, ht, hb = self.get_corrected_coords(x1, width, y1, height, \
+		                                               apply_to_selection, True)
+		# FIXME using local deltas this way "works" but isn't mathematically
+		# correct: scaled selections have a "null" and excentred central nineth
+		wl += 0.4 * width * self.zoom_level
+		wr -= 0.4 * width * self.zoom_level
+		ht += 0.4 * height * self.zoom_level
+		hb -= 0.4 * height * self.zoom_level
+		return wl, wr, ht, hb
+
+	def on_scroll_on_area(self, area, event):
+		# TODO https://lazka.github.io/pgi-docs/index.html#Gdk-3.0/classes/EventScroll.html#Gdk.EventScroll
+		ctrl_is_used = (event.state & Gdk.ModifierType.CONTROL_MASK) == Gdk.ModifierType.CONTROL_MASK
+		if ctrl_is_used == self._ctrl_to_zoom:
+			self._zoom_to_point(event)
+		else:
+			self.add_deltas(event.delta_x, event.delta_y, 10)
 
 	def on_scrollbar_value_change(self, scrollbar):
-		self.correct_coords(self.h_scrollbar.get_value(), self.v_scrollbar.get_value())
+		self.correct_coords(self._h_scrollbar.get_value(), self._v_scrollbar.get_value())
 		self.update()
 
 	def add_deltas(self, delta_x, delta_y, factor):
 		wanted_x = self.scroll_x + int(delta_x * factor)
 		wanted_y = self.scroll_y + int(delta_y * factor)
 		self.correct_coords(wanted_x, wanted_y)
-		self.window.minimap.update_minimap()
+		self.window.minimap.update_minimap(False)
 
 	def correct_coords(self, wanted_x, wanted_y):
 		available_w = self.get_widget_width()
@@ -476,9 +612,9 @@ class DrawingImage(Gtk.Box):
 
 	def update_scrollbar(self, is_vertical, allocated_size, pixbuf_size, coord):
 		if is_vertical:
-			scrollbar = self.v_scrollbar
+			scrollbar = self._v_scrollbar
 		else:
-			scrollbar = self.h_scrollbar
+			scrollbar = self._h_scrollbar
 		scrollbar.set_visible(allocated_size / self.zoom_level < pixbuf_size)
 		scrollbar.set_range(0, pixbuf_size)
 		scrollbar.get_adjustment().set_page_size(allocated_size / self.zoom_level)
@@ -488,28 +624,53 @@ class DrawingImage(Gtk.Box):
 		else:
 			self.scroll_x = int(scrollbar.get_value())
 
+	def _zoom_to_point(self, event):
+		"""Zoom in or out the image in a way such that the point under the
+		pointer stays as much as possible under the pointer."""
+		event_x, event_y = self.get_event_coords(event)
+
+		zoom_delta = (event.delta_x + event.delta_y) * -4 # Arbitrary
+		self.inc_zoom_level(zoom_delta)
+
+		# Where the zoom event occurs, in terms of percentages, in the widget's
+		# reference frame. Like, "at 10% of the width, and 80% of the height".
+		proportion_w = event.x / self.get_widget_width()
+		proportion_h = event.y / self.get_widget_height()
+
+		# Size of the rectangle between the current scroll position (values of
+		# the self.scroll_* attributes) and the zoom event. Both are expressed
+		# in pixels in the reference frame of the pixbuf.
+		corner_w = event_x - self.scroll_x
+		corner_h = event_y - self.scroll_y
+
+		# Dark magic based on unsound experiments XXX
+		fake_delta_x = corner_w * (proportion_w - 0.5)
+		fake_delta_y = corner_h * (proportion_h - 0.5)
+		if zoom_delta > 0:
+			# Zooming in
+			fake_delta_x *= 0.5
+			fake_delta_y *= 0.5
+		else:
+			# Zooming out
+			fake_delta_x *= -0.1
+			fake_delta_y *= -0.1
+
+		# Updating the scroll position based on the values previously found
+		self.add_deltas(fake_delta_x, fake_delta_y, 1)
+
 	def inc_zoom_level(self, delta):
 		self.set_zoom_level((self.zoom_level * 100) + delta)
 
 	def set_zoom_level(self, level):
-		self.zoom_level = (int(level)/100)
+		normalized_zoom_level = max(min(level, 400), 20)
+		self.zoom_level = (int(normalized_zoom_level)/100)
 		self.window.minimap.update_zoom_scale(self.zoom_level)
 		self.fake_scrollbar_update()
 		self.update()
 
-	def on_opti_zoom_level(self, *args):
-		if self.zoom_is_auto:
-			self.set_opti_zoom_level()
-
 	def set_opti_zoom_level(self, *args):
 		allocated_width = self.get_widget_width()
 		allocated_height = self.get_widget_height()
-		if allocated_width == 1:
-			# XXX because self.drawing_area might be not allocated yet
-			return
-			# allocated_width = 800
-			# allocated_height = 400
-			# FIXME but that hack can't update the minimap label
 		h_ratio = allocated_width / self.get_pixbuf_width()
 		v_ratio = allocated_height / self.get_pixbuf_height()
 		opti = min(h_ratio, v_ratio) * 99 # Not 100 because some margin is cool
