@@ -16,15 +16,15 @@
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 import sys, gi
-
 gi.require_version('Gtk', '3.0')
+gi.require_version('PangoCairo', '1.0')
 from gi.repository import Gtk, Gio, GLib, Gdk
-
-from .window import DrawingWindow
-from .preferences import DrawingPrefsWindow
+from .window import DrWindow
+from .preferences import DrPrefsWindow
 
 APP_ID = 'com.github.maoschanz.drawing'
-APP_PATH = '/com/github/maoschanz/drawing/'
+APP_PATH = '/com/github/maoschanz/drawing'
+BUG_REPORT_URL = 'https://github.com/maoschanz/drawing/issues/new/choose'
 
 def main(version):
 	app = Application(version)
@@ -36,52 +36,51 @@ class Application(Gtk.Application):
 	shortcuts_window = None
 	prefs_window = None
 
-	# INITIALIZATION ###########################################################
+	############################################################################
+	# Initialization ###########################################################
 
 	def __init__(self, version):
 		super().__init__(application_id=APP_ID,
 		                 flags=Gio.ApplicationFlags.HANDLES_COMMAND_LINE)
 
-		GLib.set_application_name('Drawing') # XXX drawing ?
+		GLib.set_application_name(_("Drawing"))
 		GLib.set_prgname(APP_ID)
-		self.version = version
+		self._version = version
 		self.has_tools_in_menubar = False
 
 		self.connect('startup', self.on_startup)
 		self.register(None)
-
 		self.connect('activate', self.on_activate)
 		self.connect('command-line', self.on_cli)
 
 		self.add_main_option('version', b'v', GLib.OptionFlags.NONE,
-		                     GLib.OptionArg.NONE,
-		                     _("Print the version and display the 'about' dialog"),
-		                     None)
+		                     # Description of a command line option
+		                     GLib.OptionArg.NONE, _("Show the app version"), None)
 		self.add_main_option('new-window', b'n', GLib.OptionFlags.NONE,
+		                     # Description of a command line option
 		                     GLib.OptionArg.NONE, _("Open a new window"), None)
 		self.add_main_option('new-tab', b't', GLib.OptionFlags.NONE,
+		                     # Description of a command line option
 		                     GLib.OptionArg.NONE, _("Open a new tab"), None)
 		self.add_main_option('edit-clipboard', b'c', GLib.OptionFlags.NONE,
+		             # Description of a command line option
 		             GLib.OptionArg.NONE, _("Edit the clipboard content"), None)
-		# TODO options pour le screenshot ?
 
 		icon_theme = Gtk.IconTheme.get_default()
-		icon_theme.add_resource_path(APP_PATH + 'icons')
-		icon_theme.add_resource_path(APP_PATH + 'tools/icons')
-
-		self.connect('window-removed', self.update_windows_menu_section)
+		icon_theme.add_resource_path(APP_PATH + '/icons')
+		icon_theme.add_resource_path(APP_PATH + '/tools/icons')
 
 	def on_startup(self, *args):
 		"""Called only once, add app-wide menus and actions, and all accels."""
-		self.build_actions()
-		builder = Gtk.Builder.new_from_resource(APP_PATH + 'ui/app-menus.ui')
+		self._build_actions()
+		builder = Gtk.Builder.new_from_resource(APP_PATH + '/ui/app-menus.ui')
 		menubar_model = builder.get_object('menu-bar')
 		self.set_menubar(menubar_model)
 		if self.prefers_app_menu():
 			appmenu_model = builder.get_object('app-menu')
 			self.set_app_menu(appmenu_model)
 
-	def build_actions(self):
+	def _build_actions(self):
 		"""Add app-wide actions."""
 		self.add_action_simple('new_window', self.on_new_window, ['<Ctrl>n'])
 		self.add_action_simple('settings', self.on_prefs, None)
@@ -92,57 +91,26 @@ class Application(Gtk.Application):
 		self.add_action_simple('help', self.on_help_index, ['F1'])
 		self.add_action_simple('help_main', self.on_help_main, None)
 		self.add_action_simple('help_tools', self.on_help_tools, None)
-		self.add_action_simple('help_canvas', self.on_help_canvas, None)
+		self.add_action_simple('help_transform', self.on_help_transform, None)
 		self.add_action_simple('help_selection', self.on_help_selection, None)
+		self.add_action_simple('help_prefs', self.on_help_prefs, None)
 		self.add_action_simple('about', self.on_about, ['<Shift>F1'])
 		self.add_action_simple('quit', self.on_quit, ['<Ctrl>q'])
-		self.add_action_enum('active-window', 0, self.change_active_win)
 
-	# WINDOWS AND CLI MANAGEMENT ###############################################
-
-	def change_active_win(self, *args):
-		win_id = args[1].get_uint32()
-		for window in self.get_windows():
-			if not isinstance(window, Gtk.ApplicationWindow):
-				continue
-			elif window.get_id() is args[1].get_uint32():
-				window.present()
-		args[0].set_state( GLib.Variant('u', win_id) )
-
-	def update_windows_menu_section(self, *args):
-		# XXX trop appelée à mon goût
-		section = self.get_menubar().get_item_link(6, \
-		          Gio.MENU_LINK_SUBMENU).get_item_link(1, Gio.MENU_LINK_SECTION)
-		section.remove_all()
-		for window in self.get_windows():
-			title = window.get_title()
-			if not isinstance(window, Gtk.ApplicationWindow):
-				continue
-			elif title is None:
-				continue
-			else:
-				detailed_name = 'app.active-window(uint32 ' + \
-				                                      str(window.get_id()) + ')'
-				try:
-					title2 = title.split(' - ')
-					title2 = title2[1] + ' - ' + title2[2]
-				except Exception:
-					title2 = title
-				section.append(title2, detailed_name)
+	############################################################################
+	# Opening windows & CLI handling ###########################################
 
 	def open_window_with_content(self, gfile, get_cb):
 		"""Open a new window with an optional Gio.File as an argument. If get_cb
 		is true, the Gio.File is ignored and the picture is built from the
 		clipboard content."""
-		win = DrawingWindow(application=self)
+		win = DrWindow(application=self)
 		win.present()
 		win.init_window_content(gfile, get_cb) # this optimization has no effect
 		# because of GLib unknown magic, but should be kept anyway because the
 		# window is presented to the user regarless of loading errors, making
 		# any issue in `init_window_content` very explicit, and more likely to
 		# be reported.
-		action = self.lookup_action('active-window')
-		action.set_state( GLib.Variant('u', win.get_id()) )
 		return win
 
 	def on_activate(self, *args):
@@ -165,8 +133,11 @@ class Application(Gtk.Application):
 		options = args[1].get_options_dict()
 
 		if options.contains('version'):
-			print(_("Drawing") + ' ' + self.version)
-			self.on_about()
+			print(_("Drawing") + ' ' + self._version)
+			if self.is_beta():
+				print(_("This version isn't stable!"))
+			print()
+			print(_("Report bugs or ideas") + ' 👉️ ' + BUG_REPORT_URL)
 		elif options.contains('edit-clipboard'):
 			self.open_window_with_content(None, True)
 
@@ -186,12 +157,12 @@ class Application(Gtk.Application):
 		elif options.contains('new-window'):
 			self.on_new_window()
 			for fpath in arguments:
-				f = self.get_valid_file(args[1], fpath)
+				f = self._get_valid_file(args[1], fpath)
 				if f is not None:
 					self.open_window_with_content(f, False)
 		else: # giving files without '-n' is equivalent to giving files with '-t'
 			for fpath in arguments:
-				f = self.get_valid_file(args[1], fpath)
+				f = self._get_valid_file(args[1], fpath)
 				if f is not None:
 					win = self.props.active_window
 					if not win:
@@ -202,7 +173,8 @@ class Application(Gtk.Application):
 		# I don't even know if i should return something
 		return 0
 
-	# ACTIONS CALLBACKS ########################################################
+	############################################################################
+	# Actions callbacks ########################################################
 
 	def on_new_window(self, *args):
 		"""Action callback, opening a new window with an empty canvas."""
@@ -211,14 +183,13 @@ class Application(Gtk.Application):
 	def on_report(self, *args):
 		"""Action callback, opening a new issue on the github repo."""
 		win = self.props.active_window
-		url = 'https://github.com/maoschanz/drawing/issues/new'
-		Gtk.show_uri_on_window(win, url, Gdk.CURRENT_TIME)
+		Gtk.show_uri_on_window(win, BUG_REPORT_URL, Gdk.CURRENT_TIME)
 
 	def on_shortcuts(self, *args):
 		"""Action callback, showing the 'shortcuts' dialog."""
 		if self.shortcuts_window is not None:
 			self.shortcuts_window.destroy()
-		builder = Gtk.Builder().new_from_resource(APP_PATH + 'ui/shortcuts.ui')
+		builder = Gtk.Builder().new_from_resource(APP_PATH + '/ui/shortcuts.ui')
 		self.shortcuts_window = builder.get_object('shortcuts')
 		self.shortcuts_window.present()
 
@@ -226,53 +197,64 @@ class Application(Gtk.Application):
 		"""Action callback, showing the preferences window."""
 		if self.prefs_window is not None:
 			self.prefs_window.destroy()
-		wants_csd = not 'ssd' in self.props.active_window.decorations
-		self.prefs_window = DrawingPrefsWindow(self.is_beta(), wants_csd)
+		wants_csd = 'h' in self.props.active_window.deco_layout
+		self.prefs_window = DrPrefsWindow(self.is_beta(), wants_csd, \
+		                                                       application=self)
 		self.prefs_window.present()
 
 	def on_help_index(self, *args):
 		"""Action callback, showing the index of user help manual."""
-		self.show_help_page('')
+		self._show_help_page('')
 
 	def on_help_main(self, *args):
 		"""Action callback, showing the 'basic features' page of the user help
 		manual."""
-		self.show_help_page('/main_features')
+		self._show_help_page('/main_features')
 
 	def on_help_tools(self, *args):
-		"""Action callback, showing the 'tools' page of the user help manual."""
-		self.show_help_page('/drawing_tools')
+		"""Action callback, showing the 'classic tools' page of the user help
+		manual."""
+		self._show_help_page('/tools_classic')
 
-	def on_help_canvas(self, *args):
-		"""Action callback, showing the 'canvas and selection tools' page of the
-		user help manual."""
-		self.show_help_page('/canvas_tools')
+	def on_help_transform(self, *args):
+		"""Action callback, showing the 'transform tools' page of the user help
+		manual."""
+		self._show_help_page('/tools_transform')
 
 	def on_help_selection(self, *args):
-		"""Action callback, showing the 'selection' page of the user
-		help manual."""
-		self.show_help_page('/selection_tools')
+		"""Action callback, showing the 'selection tools' page of the user help
+		manual."""
+		self._show_help_page('/tools_selection')
 
-	def show_help_page(self, suffix):
+	def on_help_prefs(self, *args):
+		"""Action callback, showing the 'preferences' page of the user help
+		manual."""
+		self._show_help_page('/preferences')
+
+	def _show_help_page(self, suffix):
 		win = self.props.active_window
 		Gtk.show_uri_on_window(win, 'help:drawing' + suffix, Gdk.CURRENT_TIME)
 
 	def on_about(self, *args):
 		"""Action callback, showing the "about" dialog."""
 		about_dialog = Gtk.AboutDialog(transient_for=self.props.active_window,
-			copyright="© 2018-2020 Romain F. T.", authors=["Romain F. T."],
-			# To tranlators: "translate" this by your name, it will be displayed
-			# in the "about" dialog
+			copyright="© 2018-2020 Romain F. T.",
+			authors=["Romain F. T.", "Fábio Colacio"],
+			# To tranlators: "translate" this by a list of your names (one name
+			# per line), they will be displayed in the "about" dialog
 			translator_credits=_("translator-credits"),
-			# To translators: it's credits for the icons, consider that "Art
-			# Libre" is proper name
 			artists=["Tobias Bernard", "Romain F. T.",
+			# To translators: this is credits for the icons, consider that "Art
+			# Libre" is proper name
 			                       _("GNOME's \"Art Libre\" icon set authors")],
 			comments=_("A drawing application for the GNOME desktop."),
 			license_type=Gtk.License.GPL_3_0,
-			logo_icon_name=APP_ID, version=str(self.version),
+			logo_icon_name=APP_ID, version=str(self._version),
 			website='https://maoschanz.github.io/drawing/',
 			website_label=_("Official webpage"))
+		bug_report_btn = Gtk.LinkButton(halign=Gtk.Align.CENTER, visible=True, \
+		                    label=_("Report bugs or ideas"), uri=BUG_REPORT_URL)
+		# about_dialog.get_content_area().add(bug_report_btn) # should i?
 		about_dialog.run()
 		about_dialog.destroy()
 
@@ -300,17 +282,14 @@ class Application(Gtk.Application):
 		if can_quit:
 			self.quit()
 
-	# UTILITIES ################################################################
+	############################################################################
+	# Utilities ################################################################
 
 	def is_beta(self):
 		"""Tells is the app version is even or odd, odd versions being considered
 		as unstable versions. This affects available options and the style of
 		the headerbar."""
-		version_array = self.version.split('.')
-		if (int(version_array[1]) * 5) % 10 == 5:
-			return True
-		else:
-			return False
+		return (int(self._version.split('.')[1]) * 5) % 10 == 5
 
 	def add_action_simple(self, action_name, callback, shortcuts):
 		action = Gio.SimpleAction.new(action_name, None)
@@ -325,14 +304,8 @@ class Application(Gtk.Application):
 		action.connect('change-state', callback)
 		self.add_action(action)
 
-	def add_action_enum(self, action_name, default, callback):
-		# XXX attention, celle-ci fonctionne avec un int, pas une string
-		action = Gio.SimpleAction().new_stateful(action_name, \
-		             GLib.VariantType.new('u'), GLib.Variant('u', default))
-		action.connect('change-state', callback)
-		self.add_action(action)
-
-	def get_valid_file(self, app, path):
+	def _get_valid_file(self, app, path):
+		"""Creates a GioFile object if the path corresponds to an image."""
 		try:
 			f = app.create_file_for_arg(path)
 			if 'image/' in f.query_info('standard::*', \
@@ -343,6 +316,7 @@ class Application(Gtk.Application):
 		except:
 			err = _("Error opening this file. Did you mean %s ?")
 			command = "\n\tflatpak run --file-forwarding {0} @@ {1} @@\n"
+			# TODO can happen without flatpak
 			command = command.format(APP_ID, path)
 			print(err % command)
 			return None
